@@ -3,6 +3,8 @@
 
 """Command-line support for coverage.py."""
 
+from __future__ import annotations
+
 import glob
 import optparse     # pylint: disable=deprecated-module
 import os
@@ -12,17 +14,20 @@ import sys
 import textwrap
 import traceback
 
+from typing import cast, Any, List, NoReturn, Optional, Tuple
+
 import coverage
 from coverage import Coverage
 from coverage import env
-from coverage.collector import CTracer
+from coverage.collector import HAS_CTRACER
 from coverage.config import CoverageConfig
 from coverage.control import DEFAULT_DATAFILE
 from coverage.data import combinable_files, debug_data_file
-from coverage.debug import info_formatter, info_header, short_stack
+from coverage.debug import info_header, short_stack, write_formatted_info
 from coverage.exceptions import _BaseCoverageException, _ExceptionDuringRun, NoSource
 from coverage.execfile import PyRunner
 from coverage.results import Numbers, should_fail_under
+from coverage.version import __url__
 
 # When adding to this file, alphabetization is important.  Look for
 # "alphabetize" comments throughout.
@@ -96,6 +101,10 @@ class Opts:
         '', '--fail-under', action='store', metavar="MIN", type="float",
         help="Exit with a status of 2 if the total coverage is less than MIN.",
     )
+    format = optparse.make_option(
+        '', '--format', action='store', metavar="FORMAT",
+        help="Output format, either text (default), markdown, or total.",
+    )
     help = optparse.make_option(
         '-h', '--help', action='store_true',
         help="Get help on this command.",
@@ -231,10 +240,9 @@ class CoverageOptionParser(optparse.OptionParser):
 
     """
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(
-            add_help_option=False, *args, **kwargs
-            )
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        kwargs["add_help_option"] = False
+        super().__init__(*args, **kwargs)
         self.set_defaults(
             # Keep these arguments alphabetized by their names.
             action=None,
@@ -247,6 +255,7 @@ class CoverageOptionParser(optparse.OptionParser):
             debug=None,
             directory=None,
             fail_under=None,
+            format=None,
             help=None,
             ignore_errors=None,
             include=None,
@@ -267,7 +276,7 @@ class CoverageOptionParser(optparse.OptionParser):
             timid=None,
             title=None,
             version=None,
-            )
+        )
 
         self.disable_interspersed_args()
 
@@ -275,19 +284,19 @@ class CoverageOptionParser(optparse.OptionParser):
         """Used to stop the optparse error handler ending the process."""
         pass
 
-    def parse_args_ok(self, args=None, options=None):
+    def parse_args_ok(self, args: List[str]) -> Tuple[bool, Optional[optparse.Values], List[str]]:
         """Call optparse.parse_args, but return a triple:
 
         (ok, options, args)
 
         """
         try:
-            options, args = super().parse_args(args, options)
+            options, args = super().parse_args(args)
         except self.OptionParserError:
-            return False, None, None
+            return False, None, []
         return True, options, args
 
-    def error(self, msg):
+    def error(self, msg: str) -> NoReturn:
         """Override optparse.error so sys.exit doesn't get called."""
         show_help(msg)
         raise self.OptionParserError
@@ -296,7 +305,7 @@ class CoverageOptionParser(optparse.OptionParser):
 class GlobalOptionParser(CoverageOptionParser):
     """Command-line parser for coverage.py global option arguments."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
 
         self.add_options([
@@ -308,14 +317,19 @@ class GlobalOptionParser(CoverageOptionParser):
 class CmdOptionParser(CoverageOptionParser):
     """Parse one of the new-style commands for coverage.py."""
 
-    def __init__(self, action, options, defaults=None, usage=None, description=None):
+    def __init__(
+        self,
+        action: str,
+        options: List[optparse.Option],
+        description: str,
+        usage: Optional[str] = None,
+    ):
         """Create an OptionParser for a coverage.py command.
 
         `action` is the slug to put into `options.action`.
         `options` is a list of Option's for the command.
-        `defaults` is a dict of default value for options.
-        `usage` is the usage string to display in help.
         `description` is the description of the command, for the help text.
+        `usage` is the usage string to display in help.
 
         """
         if usage:
@@ -324,18 +338,18 @@ class CmdOptionParser(CoverageOptionParser):
             usage=usage,
             description=description,
         )
-        self.set_defaults(action=action, **(defaults or {}))
+        self.set_defaults(action=action)
         self.add_options(options)
         self.cmd = action
 
-    def __eq__(self, other):
+    def __eq__(self, other: str) -> bool:       # type: ignore[override]
         # A convenience equality, so that I can put strings in unit test
         # results, and they will compare equal to objects.
         return (other == f"<CmdOptionParser:{self.cmd}>")
 
-    __hash__ = None     # This object doesn't need to be hashed.
+    __hash__ = None         # type: ignore[assignment]
 
-    def get_prog_name(self):
+    def get_prog_name(self) -> str:
         """Override of an undocumented function in optparse.OptionParser."""
         program_name = super().get_prog_name()
 
@@ -352,7 +366,7 @@ GLOBAL_ARGS = [
     Opts.debug,
     Opts.help,
     Opts.rcfile,
-    ]
+]
 
 COMMANDS = {
     'annotate': CmdOptionParser(
@@ -381,8 +395,8 @@ COMMANDS = {
             ] + GLOBAL_ARGS,
         usage="[options] <path1> <path2> ... <pathN>",
         description=(
-            "Combine data from multiple coverage files collected " +
-            "with 'run -p'.  The combined results are written to a single " +
+            "Combine data from multiple coverage files. " +
+            "The combined results are written to a single " +
             "file representing the union of the data. The positional " +
             "arguments are data files or directories containing data files. " +
             "If no paths are provided, data files in the default data file's " +
@@ -400,7 +414,8 @@ COMMANDS = {
                 "'data' to show a summary of the collected data; " +
                 "'sys' to show installation information; " +
                 "'config' to show the configuration; " +
-                "'premain' to show what is calling coverage."
+                "'premain' to show what is calling coverage; " +
+                "'pybehave' to show internal flags describing Python behavior."
         ),
     ),
 
@@ -472,7 +487,7 @@ COMMANDS = {
             Opts.output_lcov,
             Opts.omit,
             Opts.quiet,
-        ] + GLOBAL_ARGS,
+            ] + GLOBAL_ARGS,
         usage="[options] [modules]",
         description="Generate an LCOV report of coverage results.",
     ),
@@ -483,6 +498,7 @@ COMMANDS = {
             Opts.contexts,
             Opts.input_datafile,
             Opts.fail_under,
+            Opts.format,
             Opts.ignore_errors,
             Opts.include,
             Opts.omit,
@@ -535,7 +551,11 @@ COMMANDS = {
 }
 
 
-def show_help(error=None, topic=None, parser=None):
+def show_help(
+    error: Optional[str] = None,
+    topic: Optional[str] = None,
+    parser: Optional[optparse.OptionParser] = None,
+) -> None:
     """Display an error message, or the named topic."""
     assert error or topic or parser
 
@@ -555,8 +575,9 @@ def show_help(error=None, topic=None, parser=None):
             program_name = program_name[:-len(auto_suffix)]
 
     help_params = dict(coverage.__dict__)
+    help_params["__url__"] = __url__
     help_params['program_name'] = program_name
-    if CTracer is not None:
+    if HAS_CTRACER:
         help_params['extension_modifier'] = 'with C extension'
     else:
         help_params['extension_modifier'] = 'without C extension'
@@ -568,6 +589,7 @@ def show_help(error=None, topic=None, parser=None):
         print(parser.format_help().strip())
         print()
     else:
+        assert topic is not None
         help_msg = textwrap.dedent(HELP_TOPICS.get(topic, '')).strip()
         if help_msg:
             print(help_msg.format(**help_params))
@@ -582,11 +604,11 @@ OK, ERR, FAIL_UNDER = 0, 1, 2
 class CoverageScript:
     """The command-line interface to coverage.py."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.global_option = False
-        self.coverage = None
+        self.coverage: Coverage
 
-    def command_line(self, argv):
+    def command_line(self, argv: List[str]) -> int:
         """The bulk of the command line interface to coverage.py.
 
         `argv` is the argument list to process.
@@ -601,6 +623,7 @@ class CoverageScript:
 
         # The command syntax we parse depends on the first argument.  Global
         # switch syntax always starts with an option.
+        parser: Optional[optparse.OptionParser]
         self.global_option = argv[0].startswith('-')
         if self.global_option:
             parser = GlobalOptionParser()
@@ -614,6 +637,7 @@ class CoverageScript:
         ok, options, args = parser.parse_args_ok(argv)
         if not ok:
             return ERR
+        assert options is not None
 
         # Handle help and version.
         if self.do_help(options, args, parser):
@@ -647,7 +671,7 @@ class CoverageScript:
             check_preimported=True,
             context=options.context,
             messages=not options.quiet,
-            )
+        )
 
         if options.action == "debug":
             return self.do_debug(args)
@@ -674,7 +698,7 @@ class CoverageScript:
             omit=omit,
             include=include,
             contexts=contexts,
-            )
+        )
 
         # We need to be able to import from the current directory, because
         # plugins may try to, for example, to read Django settings.
@@ -690,8 +714,9 @@ class CoverageScript:
                 skip_covered=options.skip_covered,
                 skip_empty=options.skip_empty,
                 sort=options.sort,
+                output_format=options.format,
                 **report_args
-                )
+            )
         elif options.action == "annotate":
             self.coverage.annotate(directory=options.directory, **report_args)
         elif options.action == "html":
@@ -703,25 +728,25 @@ class CoverageScript:
                 show_contexts=options.show_contexts,
                 title=options.title,
                 **report_args
-                )
+            )
         elif options.action == "xml":
             total = self.coverage.xml_report(
                 outfile=options.outfile,
                 skip_empty=options.skip_empty,
                 **report_args
-                )
+            )
         elif options.action == "json":
             total = self.coverage.json_report(
                 outfile=options.outfile,
                 pretty_print=options.pretty_print,
                 show_contexts=options.show_contexts,
                 **report_args
-                )
+            )
         elif options.action == "lcov":
             total = self.coverage.lcov_report(
                 outfile=options.outfile,
                 **report_args
-                )
+            )
         else:
             # There are no other possible actions.
             raise AssertionError
@@ -734,8 +759,8 @@ class CoverageScript:
             if options.precision is not None:
                 self.coverage.set_option("report:precision", options.precision)
 
-            fail_under = self.coverage.get_option("report:fail_under")
-            precision = self.coverage.get_option("report:precision")
+            fail_under = cast(float, self.coverage.get_option("report:fail_under"))
+            precision = cast(int, self.coverage.get_option("report:precision"))
             if should_fail_under(total, fail_under, precision):
                 msg = "total of {total} is less than fail-under={fail_under:.{p}f}".format(
                     total=Numbers(precision=precision).display_covered(total),
@@ -747,7 +772,12 @@ class CoverageScript:
 
         return OK
 
-    def do_help(self, options, args, parser):
+    def do_help(
+        self,
+        options: optparse.Values,
+        args: List[str],
+        parser: optparse.OptionParser,
+    ) -> bool:
         """Deal with help requests.
 
         Return True if it handled the request, False if not.
@@ -764,9 +794,9 @@ class CoverageScript:
         if options.action == "help":
             if args:
                 for a in args:
-                    parser = COMMANDS.get(a)
-                    if parser:
-                        show_help(parser=parser)
+                    parser_maybe = COMMANDS.get(a)
+                    if parser_maybe is not None:
+                        show_help(parser=parser_maybe)
                     else:
                         show_help(topic=a)
             else:
@@ -780,7 +810,7 @@ class CoverageScript:
 
         return False
 
-    def do_run(self, options, args):
+    def do_run(self, options: optparse.Values, args: List[str]) -> int:
         """Implementation of 'coverage run'."""
 
         if not args:
@@ -788,7 +818,7 @@ class CoverageScript:
                 # Specified -m with nothing else.
                 show_help("No module specified for -m")
                 return ERR
-            command_line = self.coverage.get_option("run:command_line")
+            command_line = cast(str, self.coverage.get_option("run:command_line"))
             if command_line is not None:
                 args = shlex.split(command_line)
                 if args and args[0] in {"-m", "--module"}:
@@ -839,36 +869,32 @@ class CoverageScript:
 
         return OK
 
-    def do_debug(self, args):
+    def do_debug(self, args: List[str]) -> int:
         """Implementation of 'coverage debug'."""
 
         if not args:
-            show_help("What information would you like: config, data, sys, premain?")
+            show_help("What information would you like: config, data, sys, premain, pybehave?")
             return ERR
         if args[1:]:
             show_help("Only one topic at a time, please")
             return ERR
 
-        if args[0] == 'sys':
-            sys_info = self.coverage.sys_info()
-            print(info_header("sys"))
-            for line in info_formatter(sys_info):
-                print(f" {line}")
-        elif args[0] == 'data':
+        if args[0] == "sys":
+            write_formatted_info(print, "sys", self.coverage.sys_info())
+        elif args[0] == "data":
             print(info_header("data"))
             data_file = self.coverage.config.data_file
             debug_data_file(data_file)
             for filename in combinable_files(data_file):
                 print("-----")
                 debug_data_file(filename)
-        elif args[0] == 'config':
-            print(info_header("config"))
-            config_info = sorted(self.coverage.config.__dict__.items())
-            for line in info_formatter(config_info):
-                print(f" {line}")
+        elif args[0] == "config":
+            write_formatted_info(print, "config", self.coverage.config.debug_info())
         elif args[0] == "premain":
             print(info_header("premain"))
             print(short_stack())
+        elif args[0] == "pybehave":
+            write_formatted_info(print, "pybehave", env.debug_info())
         else:
             show_help(f"Don't know what you mean by {args[0]!r}")
             return ERR
@@ -876,7 +902,7 @@ class CoverageScript:
         return OK
 
 
-def unshell_list(s):
+def unshell_list(s: str) -> Optional[List[str]]:
     """Turn a command-line argument into a list."""
     if not s:
         return None
@@ -890,7 +916,7 @@ def unshell_list(s):
     return s.split(',')
 
 
-def unglob_args(args):
+def unglob_args(args: List[str]) -> List[str]:
     """Interpret shell wildcards for platforms that need it."""
     if env.WINDOWS:
         globbed = []
@@ -936,7 +962,7 @@ HELP_TOPICS = {
 }
 
 
-def main(argv=None):
+def main(argv: Optional[List[str]] = None) -> Optional[int]:
     """The main entry point to coverage.py.
 
     This is installed as the script entry point.
@@ -974,7 +1000,9 @@ if _profile:                                                # pragma: debugging
     from ox_profile.core.launchers import SimpleLauncher    # pylint: disable=import-error
     original_main = main
 
-    def main(argv=None):                                    # pylint: disable=function-redefined
+    def main(                                               # pylint: disable=function-redefined
+        argv: Optional[List[str]] = None,
+    ) -> Optional[int]:
         """A wrapper around main that profiles."""
         profiler = SimpleLauncher.launch()
         try:
