@@ -3,10 +3,21 @@
 
 """Determine facts about the environment."""
 
+from __future__ import annotations
+
 import os
 import platform
 import sys
 
+from typing import Any, Iterable, Tuple
+
+# debug_info() at the bottom wants to show all the globals, but not imports.
+# Grab the global names here to know which names to not show. Nothing defined
+# above this line will be in the output.
+_UNINTERESTING_GLOBALS = list(globals())
+# These names also shouldn't be shown.
+_UNINTERESTING_GLOBALS += ["PYBEHAVIOR", "debug_info"]
+
 # Operating systems.
 WINDOWS = sys.platform == "win32"
 LINUX = sys.platform.startswith("linux")
@@ -15,15 +26,13 @@ OSX = sys.platform == "darwin"
 # Python implementations.
 CPYTHON = (platform.python_implementation() == "CPython")
 PYPY = (platform.python_implementation() == "PyPy")
-JYTHON = (platform.python_implementation() == "Jython")
-IRONPYTHON = (platform.python_implementation() == "IronPython")
 
 # Python versions. We amend version_info with one more value, a zero if an
 # official version, or 1 if built from source beyond an official version.
 PYVERSION = sys.version_info + (int(platform.python_version()[-1] == "+"),)
 
 if PYPY:
-    PYPYVERSION = sys.pypy_version_info
+    PYPYVERSION = sys.pypy_version_info         # type: ignore[attr-defined]
 
 # Python behavior.
 class PYBEHAVIOR:
@@ -39,20 +48,22 @@ class PYBEHAVIOR:
     else:
         optimize_if_debug = not pep626
 
-    # Is "if not __debug__" optimized away?
-    optimize_if_not_debug = (not PYPY) and (PYVERSION >= (3, 7, 0, 'alpha', 4))
+    # Is "if not __debug__" optimized away? The exact details have changed
+    # across versions.
     if pep626:
-        optimize_if_not_debug = False
-    if PYPY:
-        optimize_if_not_debug = True
-
-    # Is "if not __debug__" optimized away even better?
-    optimize_if_not_debug2 = (not PYPY) and (PYVERSION >= (3, 8, 0, 'beta', 1))
-    if pep626:
-        optimize_if_not_debug2 = False
-
-    # Yet another way to optimize "if not __debug__"?
-    optimize_if_not_debug3 = (PYPY and PYVERSION >= (3, 8))
+        optimize_if_not_debug = 1
+    elif PYPY:
+        if PYVERSION >= (3, 9):
+            optimize_if_not_debug = 2
+        elif PYVERSION[:2] == (3, 8):
+            optimize_if_not_debug = 3
+        else:
+            optimize_if_not_debug = 1
+    else:
+        if PYVERSION >= (3, 8, 0, 'beta', 1):
+            optimize_if_not_debug = 2
+        else:
+            optimize_if_not_debug = 1
 
     # Can co_lnotab have negative deltas?
     negative_lnotab = not (PYPY and PYPYVERSION < (7, 2))
@@ -65,25 +76,47 @@ class PYBEHAVIOR:
     # does the finally jump back to the break/continue/return (3.8) to do the
     # work?
     finally_jumps_back = ((3, 8) <= PYVERSION < (3, 10))
+    if PYPY and PYPYVERSION < (7, 3, 7):
+        finally_jumps_back = False
 
     # When a function is decorated, does the trace function get called for the
     # @-line and also the def-line (new behavior in 3.8)? Or just the @-line
     # (old behavior)?
-    trace_decorated_def = (CPYTHON and PYVERSION >= (3, 8))
+    trace_decorated_def = (
+        (PYVERSION >= (3, 8)) and
+        (CPYTHON or (PYVERSION > (3, 8)) or (PYPYVERSION > (7, 3, 9)))
+    )
+
+    # Functions are no longer claimed to start at their earliest decorator even though
+    # the decorators are traced?
+    def_ast_no_decorator = (PYPY and PYVERSION >= (3, 9))
+
+    # CPython 3.11 now jumps to the decorator line again while executing
+    # the decorator.
+    trace_decorator_line_again = (CPYTHON and PYVERSION > (3, 11, 0, 'alpha', 3, 0))
 
     # Are while-true loops optimized into absolute jumps with no loop setup?
     nix_while_true = (PYVERSION >= (3, 8))
 
-    # Python 3.9a1 made sys.argv[0] and other reported files absolute paths.
-    report_absolute_files = (PYVERSION >= (3, 9))
+    # CPython 3.9a1 made sys.argv[0] and other reported files absolute paths.
+    report_absolute_files = (
+        (CPYTHON or (PYPY and PYPYVERSION >= (7, 3, 10)))
+        and PYVERSION >= (3, 9)
+    )
 
     # Lines after break/continue/return/raise are no longer compiled into the
     # bytecode.  They used to be marked as missing, now they aren't executable.
-    omit_after_jump = pep626
+    omit_after_jump = (
+        pep626
+        or (PYPY and PYVERSION >= (3, 9) and PYPYVERSION >= (7, 3, 12))
+    )
 
     # PyPy has always omitted statements after return.
     omit_after_return = omit_after_jump or PYPY
 
+    # Optimize away unreachable try-else clauses.
+    optimize_unreachable_try_else = pep626
+
     # Modules used to have firstlineno equal to the line number of the first
     # real line of code.  Now they always start at 1.
     module_firstline_1 = pep626
@@ -100,10 +133,9 @@ class PYBEHAVIOR:
     # Some words are keywords in some places, identifiers in other places.
     soft_keywords = (PYVERSION >= (3, 10))
 
-    # CPython 3.11 now jumps to the decorator line again while executing
-    # the decorator.
-    trace_decorator_line_again = (PYVERSION > (3, 11, 0, 'alpha', 3, 0))
-
+    # Modules start with a line numbered zero. This means empty modules have
+    # only a 0-number line, which is ignored, giving a truly empty module.
+    empty_is_empty = (PYVERSION >= (3, 11, 0, 'beta', 4))
 
 # Coverage.py specifics.
 
@@ -115,14 +147,18 @@ METACOV = os.getenv('COVERAGE_COVERAGE', '') != ''
 
 # Are we running our test suite?
 # Even when running tests, you can use COVERAGE_TESTING=0 to disable the
-# test-specific behavior like contracts.
+# test-specific behavior like AST checking.
 TESTING = os.getenv('COVERAGE_TESTING', '') == 'True'
 
-# Environment COVERAGE_NO_CONTRACTS=1 can turn off contracts while debugging
-# tests to remove noise from stack traces.
-# $set_env.py: COVERAGE_NO_CONTRACTS - Disable PyContracts to simplify stack traces.
-USE_CONTRACTS = (
-    TESTING
-    and not bool(int(os.environ.get("COVERAGE_NO_CONTRACTS", 0)))
-    and (PYVERSION < (3, 11))
-)
+
+def debug_info() -> Iterable[Tuple[str, Any]]:
+    """Return a list of (name, value) pairs for printing debug information."""
+    info = [
+        (name, value) for name, value in globals().items()
+        if not name.startswith("_") and name not in _UNINTERESTING_GLOBALS
+    ]
+    info += [
+        (name, value) for name, value in PYBEHAVIOR.__dict__.items()
+        if not name.startswith("_")
+    ]
+    return sorted(info)
